import pandas as pd


def align_features_and_targets(features: pd.DataFrame, targets: pd.DataFrame):
    all = features.join(targets, how="inner")

    features = all[features.columns]
    targets = all[targets.columns]
    return features, targets, all


def combine_features(features_list: list) -> pd.DataFrame:
    """
    combine array of dataframes containing features
    joining on indexes
    """

    features_list = [f for f in features_list if len(f) > 0]

    if len(features_list) == 0:
        return features_list

    results = pd.concat(features_list, join='inner', axis=1)
    return results


def _get_lag_col_name(feature_col: str, lag_len: int):
    return f"{feature_col}-lag_{lag_len}"


def _apply_lag_full_col_name(features: pd.DataFrame, lag_feature_full_name: str, lag_len: str):
    features[_get_lag_col_name(feature_col=lag_feature_full_name, lag_len=lag_len)] = features[
        [lag_feature_full_name]].shift(
        lag_len)
    return features


def create_feature_lags(features: pd.DataFrame, targets: pd.DataFrame, lag_feature_start_with: str, lag_len: str):
    """
        Note: lags means multivariate!!!!
    """
    cols = features.columns[features.columns.str.startswith(lag_feature_start_with)]

    for col in cols:
        features = _apply_lag_full_col_name(features=features, lag_feature_full_name=col, lag_len=lag_len)

    features = features.iloc[lag_len:]
    targets = targets.iloc[lag_len:]
    return features, targets


def _drop_col_if_exists(data: pd.DataFrame, col_name: str):
    if col_name in data.columns:
        _ = data.pop(col_name)
    return data


def remove_ohlc_columns_if_exists(data: pd.DataFrame):
    _drop_col_if_exists(data=data, col_name="Open")
    _drop_col_if_exists(data=data, col_name="High")
    _drop_col_if_exists(data=data, col_name="Low")
    _drop_col_if_exists(data=data, col_name="Close")
    return data


def drop_na(features: pd.DataFrame, targets: pd.DataFrame):
    # print(features.describe())
    print(f"row loss due to NaN: {len(features) - len(features.dropna())}")
    features = features.dropna()

    features, targets, all = align_features_and_targets(features=features, targets=targets)
    return features, targets, all


def add_ohlc(data: pd.DataFrame, ohlc: pd.DataFrame):
    ohlc_cols = ["Open", "High", "Low", "Close"]
    remove_ohlc_columns_if_exists(data=data)
    result = combine_features(features_list=[ohlc, data])

    return result



import pytest
from moosir_feature.news_feature.app_settings import Settings
from moosir_feature.news_feature.news_manager import *
import os

TEMP_FILE = "temp.h5"


@pytest.fixture()
def set_up():
    # set up
    if os.path.exists(TEMP_FILE):
        os.remove(TEMP_FILE)

    yield "set_up"

    # tear down
    if os.path.exists(TEMP_FILE):
        os.remove(TEMP_FILE)


def test_get_news_month(set_up):
    # arrange
    settings = Settings()

    dal = H5DataAccessLayer(file_path=TEMP_FILE, data_provider=settings.data_provider)
    manager = NewsFeatureManager(dal=dal, data_provider_tz=settings.data_provider_tz, countries=settings.countries)

    month = 1
    year = 2020

    # act
    manager.save_news_months(months=[1], year=2020)
    result = manager.get_news_month(month=month, year=year)
    # result = manager.get_news()

    # assert
    assert len(result[result["Timestamp"].dt.year != year]) == 0
    assert len(result[result["Timestamp"].dt.month != month]) == 0
    assert len(result[~result["currency"].isin(settings.countries)]) == 0
    assert len(result[result["importance"] != "high"]) == 0
    print(result)


import pytest

from moosir_feature.transformers.indicators.feature_calculator import *
from moosir_feature.transformers.indicators.tech_indicators import *


@pytest.fixture
def ohlc_prices():
    return pd.DataFrame(data=np.random.rand(10, 4), columns=["Open", "High", "Low", "Close"])


def test_create_operators():
    # arrange
    indicator_names = [RsiOperator.__name__,
                       BollingerBandOperator.__name__,
                       AtrOperator.__name__,
                       MaOperator.__name__,
                       MomentomOperator.__name__,
                       WmaOperator.__name__,
                       AdxOperator.__name__,
                       ]
    look_back_periods = [10, 20]
    prefix = "random"

    # act
    result = create_operators(look_back_periods=look_back_periods,
                              prefix=prefix,
                              tech_indicator_names=indicator_names)

    # assert
    assert len(result) == len(look_back_periods) * len(indicator_names)


def test_apply_historical_return(ohlc_prices):
    # arrange
    periods = [3, 4]
    magnifier_multiplier = 2
    expected_operators = [ReturnsOperator.__name__]
    original_ohlc = ohlc_prices.copy()

    # act
    result = apply_historical_return(ohlc_price=ohlc_prices,
                                     periods=periods,
                                     magnifier_multiplier=magnifier_multiplier)

    # assert
    assert_operators_results(operator_names=expected_operators,
                             look_back_periods=periods,
                             original_ohlc=original_ohlc,
                             result=result,
                             expected_columns=["Ind-Returns"])


def test_apply_forward_max_return_and_price(ohlc_prices):
    # arrange
    periods = [3, 4]
    expected_operators = [ForwardMaxPriceAndReturn.__name__]
    original_ohlc = ohlc_prices.copy()
    expected_columns = ["Fwd-Price-Max-", "Fwd-Return-Max-"]

    # act
    result = apply_forward_max_return_and_price(ohlc_price=ohlc_prices,
                                                periods=periods)

    # assert
    assert len(result.columns) == 4 + 2 * len(expected_operators) * len(periods)

    for col in expected_columns:
        assert len(result.filter(like=col).columns) == len(periods)

    result_ohlc = result[["Open", "High", "Low", "Close"]]
    pd.testing.assert_frame_equal(result_ohlc, original_ohlc)


def test_apply_forward_min_return_and_price(ohlc_prices):
    # arrange
    periods = [3, 4]
    expected_operators = [ForwardMinPriceAndReturn.__name__]
    original_ohlc = ohlc_prices.copy()
    expected_columns = ["Fwd-Price-Min-", "Fwd-Return-Min-"]

    # act
    result = apply_forward_min_price_and_return(ohlc_price=ohlc_prices,
                                                periods=periods)

    # assert
    assert len(result.columns) == 4 + 2 * len(expected_operators) * len(periods)

    for col in expected_columns:
        assert len(result.filter(like=col).columns) == len(periods)

    result_ohlc = result[["Open", "High", "Low", "Close"]]
    pd.testing.assert_frame_equal(result_ohlc, original_ohlc)


def test_forward_highest_return(ohlc_prices):
    # arrange
    periods = [3, 4]
    original_ohlc = ohlc_prices.copy()
    expected_columns = ["Fwd-Price-Min-", "Fwd-Return-Min-",
                        "Fwd-Price-Max-", "Fwd-Return-Max-",
                        "Fwd-Return-Highest-"]

    # act
    result = apply_forward_min_price_and_return(ohlc_price=ohlc_prices,
                                                periods=periods)

    result = apply_forward_max_return_and_price(ohlc_price=result,
                                                periods=periods)

    result = apply_forward_highest_return(ohlc_price=result,
                                          periods=periods)

    # assert
    assert len(result.columns) == 4 + 4 * len(periods) + len(periods)

    for col in expected_columns:
        assert len(result.filter(like=col).columns) == len(periods)

    result_ohlc = result[["Open", "High", "Low", "Close"]]
    pd.testing.assert_frame_equal(result_ohlc, original_ohlc)


def test_apply_technical(ohlc_prices):
    # arrange
    indicators = [RsiOperator.__name__,
                  VilliamrOperator.__name__
                  ]
    original_ohlc = ohlc_prices.copy()
    look_back_periods = [3, 4]

    # act
    result, _ = apply_technical_indicators(ohlc_price=ohlc_prices,
                                           look_back_periods=look_back_periods,
                                           tech_indicator_names=indicators)

    # assert
    assert_operators_results(operator_names=indicators,
                             look_back_periods=look_back_periods,
                             original_ohlc=original_ohlc,
                             result=result,
                             expected_columns=["Rsi", "Willr"])


def test_apply_lags(ohlc_prices):
    # arrange
    periods = [3, 4]
    magnifier_multiplier = 2
    lag_periods = [2]
    return_period_for_lag = [3]

    # act
    _ = apply_historical_return(ohlc_price=ohlc_prices,
                                periods=periods,
                                magnifier_multiplier=magnifier_multiplier)

    # act

    lag_result = apply_lags(ohlc_price=ohlc_prices, lag_periods=lag_periods, ind_periods=return_period_for_lag)

    # assert
    result_cols = lag_result.columns
    #
    assert "Lag-Ind-Returns-3-T-0002" in result_cols
    assert "Lag-Ind-Returns-3-T-0003" not in result_cols


def assert_operators_results(operator_names, look_back_periods, original_ohlc, result, expected_columns):
    assert len(result.columns) == 4 + len(operator_names) * len(look_back_periods)

    for col in expected_columns:
        assert len(result.filter(like=col).columns) == len(look_back_periods)

    result_ohlc = result[["Open", "High", "Low", "Close"]]
    pd.testing.assert_frame_equal(result_ohlc, original_ohlc)



import pandas as pd
import numpy as np
import pytest
from sklearn.dummy import DummyRegressor

from moosir_feature.model_validations.benchmarking import RandomModel, run_benchmarking, NaiveModel

from moosir_feature.model_validations.model_validator import CustomTsCv


# @pytest.mark.skip()
def test_random_with_cv():
    # arrange
    sample_n = 100
    feature_n = 1

    feature_cols = [f"f_{i}" for i in range(feature_n)]
    target_col = "target"
    features = pd.DataFrame(data=np.arange(0, sample_n * feature_n).reshape(sample_n, -1),
                            columns=feature_cols,
                            index=pd.date_range("2000/01/01", periods=sample_n))
    targets = pd.DataFrame(data=np.arange(0, sample_n * 1).reshape(sample_n, -1),
                           columns=[target_col],
                           index=pd.date_range("2000/01/01", periods=sample_n))

    # arrange model and cv
    min = targets.min()
    max = targets.max()

    models = [RandomModel(min=min, max=max), NaiveModel(targets=targets.copy(), look_back_len=1)]

    cv = CustomTsCv(train_n=10, test_n=3, sample_n=sample_n, train_shuffle_block_size=None)

    metrics = ['neg_mean_absolute_percentage_error',
               'neg_mean_absolute_error',
               'binary_returns_avg',
               'binary_returns']

    # act
    result = run_benchmarking(models=models, targets=targets, features=features, cv=cv, metrics=metrics)

    # assert
    print(result)



import pandas as pd
import numpy as np
from sklearn.metrics import get_scorer, SCORERS, make_scorer

def binary_returns_metric_fn(y_true: pd.DataFrame, y_pred: np.array):
    assert len(y_true.columns) == 1
    assert np.all((y_pred == 0) | (y_pred == 1)), "predictions must be either 0 or 1"

    temp = y_true.copy()
    temp["_preds"] = y_pred
    score = (temp[temp.columns[0]] * temp["_preds"]).sum()

    return score

def binary_returns_avg_metric_fn(y_true: pd.DataFrame, y_pred: np.array):
    assert len(y_true.columns) == 1
    assert np.all((y_pred == 0) | (y_pred == 1)), "predictions must be either 0 or 1"

    temp = y_true.copy()
    temp["_preds"] = y_pred
    score = (temp[temp.columns[0]] * temp["_preds"]).sum()/len(temp)

    return score


CUSTOM_SCORERS = dict(
    binary_returns=make_scorer(binary_returns_metric_fn),
    binary_returns_avg=make_scorer(binary_returns_avg_metric_fn),
)


def get_any_metric(metrics: list):
    scorers = {}
    for metric in metrics:
        if metric in SCORERS.keys():
            scorers.update({metric: get_scorer(metric)})
        else:
            scorers.update({metric: CUSTOM_SCORERS[metric]})

    return scorers


import pandas as pd
import numpy as np

from tsfresh.feature_extraction import settings
from tsfresh.utilities.dataframe_functions import roll_time_series
from tsfresh import extract_features

TSF_ID_COL = "Tsf_Id"
FEATURES_SCHEMA_DICS = {"linear_trend_timewise": ["Trend-Pvalue", "Trend-Rvalue", "Trend-Intercept", "Trend-Slope",
                                                  "Trend-Stderr"],
                        "abs_energy": ["Abs-energy"],
                        "benford_correlation": ["Benford-correlation"],
                        "binned_entropy": ["Binned-entropybin10"],
                        "c3": ["C3-laj1", "C3-laj2", "C3-laj3"],

                        # todo: not working, exceptions
                        # "fft_aggregated": ["Centroid", "Variance", "Skew", "Kurtosis"],
                        # "agg_autocorrelation": ["Mean_lag_x", "Median_lag_x", "Var_lag_x"],
                        # "absolute_sum_of_changes": ["Absolute_sum_of_changes"],
                        # "absolute_maximum": ["Absolute-maximum"],
                        }

FEATURE_PREFIX = "TS"


def _get_feature_col_name(prefix: str, win_len: int, orig_col_name: str):
    return f"{prefix}{'' if prefix == '' else '-'}{orig_col_name}-{win_len}"


def get_feature_col_full_name(feature_name: str, win_len: int):
    result = [_get_feature_col_name(prefix=FEATURE_PREFIX, win_len=win_len, orig_col_name=k)
              for k in
              FEATURES_SCHEMA_DICS[feature_name]]
    return result


def get_feature_col_by_subtype_name(feature_name: str, win_len: int, subtype_name: str, is_fwd: bool, fwd_len: -1):
    if is_fwd:
        alls = get_feature_fwd_full_name(feature_name=feature_name, win_len=win_len, fwd_len=fwd_len)
    else:
        alls = get_feature_col_full_name(feature_name=feature_name, win_len=win_len)

    # todo: contain is wrong here, proper break by '-' and has Fwd, lag logic
    for f in alls:
        if subtype_name in f:
            return f


def get_feature_lag_full_name(feature_name: str, win_len: int, lag_len: int):
    col_names = get_feature_col_full_name(feature_name=feature_name, win_len=win_len)
    results = [f"{col_name}_Lag-{lag_len}" for col_name in col_names]
    return results


def get_feature_fwd_full_name(feature_name: str, win_len: int, fwd_len: int):
    col_names = get_feature_col_full_name(feature_name=feature_name, win_len=win_len)
    results = [f"{col_name}_Fwd-{fwd_len}" for col_name in col_names]
    return results


def validate_columns(data: pd.DataFrame,
                     feature_names: list,
                     win_lens: list,
                     lag_lens: list = None,
                     forward_lens: list = None):
    assert lag_lens is None or forward_lens is None, "forwards and lags cant be not none at the same time"

    cols = data.columns
    for f in feature_names:
        for w in win_lens:
            if lag_lens:
                for lag in lag_lens:
                    col_names = get_feature_lag_full_name(feature_name=f, win_len=w, lag_len=lag)
                    for col in col_names:
                        assert col in cols, f"missing column {col}"
            elif forward_lens:
                for fwd in forward_lens:
                    col_names = get_feature_fwd_full_name(feature_name=f, win_len=w, fwd_len=fwd)
                    for col in col_names:
                        assert col in cols, f"missing column {col}"
            else:
                col_names = get_feature_col_full_name(feature_name=f, win_len=w)
                for col in col_names:
                    assert col in cols, f"missing column {col}"
    return True


def _tsf_preprocess_data(data: pd.DataFrame):
    # preprocess data
    tsf_data = data.copy()
    tsf_data[TSF_ID_COL] = 1
    tsf_data = tsf_data.reset_index()
    return tsf_data


def calc_rolling_feature(ohlc: pd.DataFrame, win_len=10, selected_features=["linear_trend_timewise"],
                         output_col_prefix=FEATURE_PREFIX, verbose=False):
    """
        applies one feature on input data rolling wind
        Note: tsfresh feature result cant be pandas per window
    """
    if verbose:
        print("#######################")
        print("prepare data for tsfresh")
        print("#######################")
    data = ohlc[["Close"]]
    assert data.index.name == "Timestamp"

    tsf_data = _tsf_preprocess_data(data=data)
    tsf_data = tsf_data.set_index("Timestamp")
    if verbose:
        print("#######################")
        print("prepare rolling windows")
        print("#######################")

    df_rolled = roll_time_series(tsf_data, column_id="Tsf_Id", max_timeshift=win_len, min_timeshift=win_len)

    # todo: need random timestamp index!!! otherwise, returns empty
    rand_indx = pd.date_range(start='1/1/2000', periods=len(df_rolled), freq="T")
    df_rolled.index = rand_indx
    df_rolled.index.name = "Timestamp"

    if verbose:
        print("#######################")
        print("calculate features")
        print("#######################")

    settings_comprehensive = settings.ComprehensiveFCParameters()

    selected_settings = {}
    selected_settings_cols = []
    for key in selected_features:
        selected_settings[key] = settings_comprehensive[key]
        selected_settings_cols = selected_settings_cols + get_feature_col_full_name(feature_name=key, win_len=win_len)

    kind_to_fc_parameters = {"Close": selected_settings}

    roll_features = extract_features(df_rolled,
                                     column_id="id",
                                     column_sort="sort",
                                     column_value="Close",
                                     kind_to_fc_parameters=kind_to_fc_parameters,
                                     show_warnings=True
                                     )
    if verbose:
        print("#######################")
        print("mapping output columns")
        print("#######################")

    roll_features.index.set_names(["Tsf_Id", "Ind"], inplace=True)
    roll_features = roll_features.swaplevel()
    roll_features = roll_features.reset_index(level=1)
    _ = roll_features.pop("Tsf_Id")

    roll_features["Timestamp"] = data.index[roll_features.index.values]
    roll_features = roll_features.reset_index().set_index("Timestamp")
    _ = roll_features.pop("Ind")

    roll_features.columns = selected_settings_cols

    roll_features = roll_features.join(data)

    assert len(roll_features) + win_len == len(
        data), f"some data missed during calculation. lengths: input {len(data)}, result {len(roll_features)}, window: {win_len} "

    return roll_features


if __name__ == '__main__':
    DATA_PATH = r'C:\datalake-train\moosir\preproc\data\01_raw\close_market_data.csv'
    data = pd.read_csv(DATA_PATH, index_col="Timestamp", parse_dates=True)

    data = data.iloc[2000:2200]
    selected_features = ["linear_trend_timewise", "fft_aggregated"]
    res = calc_rolling_feature(data=data,
                               selected_features=selected_features)

    print(res)



import numpy as np
import pandas as pd

from moosir_feature.transformers.feature_cleaning.data_cleaner import *

import pytest

def test_remove_duplicates():
    # arrange
    n_samples = 15
    n_duplicates = 10

    data = pd.DataFrame(data=np.random.rand(n_samples, 1), index=pd.date_range("2020/02/01", periods=n_samples))
    data1 = pd.DataFrame(data=np.random.rand(n_duplicates, 1), index=pd.date_range("2020/02/01", periods=n_duplicates))

    input = pd.concat([data, data1])
    input.index.name = "Timestamp"

    # act
    removed_duplicates, duplicates = remove_duplicate_timestamps(data=input)

    # assert
    assert len(duplicates) == n_duplicates
    assert len(removed_duplicates) == n_samples


def test_remove_na():
    # arrange
    n_samples = 10

    data = pd.DataFrame(data=np.random.rand(n_samples, 1), index=pd.date_range("2020/02/01", periods=n_samples))
    data.iloc[0][0] = np.NaN
    data.iloc[3][0] = np.NaN

    # act
    no_nans, nan_rows = remove_na(data=data)

    # assert
    assert len(no_nans) == n_samples - 2
    assert len(nan_rows) == 2


def test_find_missing_timestamp():
    # arrange

    n_samples = 5
    freq = "D"

    data_a = pd.DataFrame(data=np.random.rand(n_samples, 1), index=pd.date_range("2022/02/01", periods=n_samples, freq=freq))
    data_b = pd.DataFrame(data=np.random.rand(n_samples, 1), index=pd.date_range("2022/02/15", periods=n_samples, freq=freq))

    data = pd.concat([data_a, data_b])
    data.index.name = "Timestamp"

    # act
    result_no_weekends = find_missing_timestamp(data=data, freq=freq, exclude_weekends=True)
    result_with_weekends = find_missing_timestamp(data=data, freq=freq, exclude_weekends=False)

    # assert
    assert len(result_no_weekends) == 9 - 3 # excluding weekends
    assert len(result_with_weekends) == 9 # including weekends


def test_check_close_match_open_next_bar():

    # arrange

    data_dic = {
        'Open': [1, 1, 2, 100],
        'Close': [1, 2, 3, 4],
    }

    data = pd.DataFrame(data=data_dic)

    # act
    result = check_close_match_open_next_bar(ohlc=data)

    # assert
    assert (result == np.array([2, 3])).all()


def test_check_high_low_values():

    # arrange
    data_dic = {
        'Open': [1, 200, 3, 4],
        'High': [10, 20, 30, 40],
        'Low': [0.1, 0.2, 0.3, 0.4],
        'Close': [1, 2, 0.03, 4],
    }

    data = pd.DataFrame(data=data_dic)

    # act
    close_problems, open_problems = check_high_low_values(ohlc=data)

    # assert
    assert (close_problems.index == np.array([2])).all()
    assert (open_problems.index == np.array([1])).all()


def test_check_high_low_values():

    # arrange
    data_dic = {
        'Open': [1, 2, 3, 4],
        'High': [2, 2, 0.2, 2],
    }

    data = pd.DataFrame(data=data_dic)

    # act
    result, violating = remove_violating_threshold(data=data, low=1, high=3)

    # assert
    assert (result.index == np.array([0, 1])).all()
    assert (violating.index == np.array([2, 3])).all()


def test_remove_weekends_bars():
    # arrange

    n_samples = 10
    freq = "D"

    data = pd.DataFrame(data=np.random.rand(n_samples, 1),
                          index=pd.date_range("2022/02/01", periods=n_samples, freq=freq))

    data.index.name = "Timestamp"

    # act
    no_weekends, weekends = remove_weekends_bars(data=data)

    # assert
    assert len(no_weekends) == 8  # excluding weekends
    assert len(weekends) == 2  # including weekends



import pandas as pd


def remove_duplicate_timestamps(data: pd.DataFrame):
    assert data.index.name == "Timestamp", "input does nt have timestamp index"

    dup_inds = data.index.duplicated(keep='first')
    duplicates = data.loc[dup_inds]
    removed_duplicates = data.loc[~dup_inds]

    return removed_duplicates, duplicates


def remove_na(data: pd.DataFrame):
    # is_nulls = pd.isnull(data).sum() if data.columns > 1 else pd.isnull(data)
    is_nulls = pd.isnull(data).sum(axis=1)

    nan_rows = data[is_nulls > 0]

    no_nans = data[is_nulls == 0]

    return no_nans, nan_rows


"""
some checks (high frequency data, tick data)
- zero or negative data
- low volume ticks
- gaps (missing data in timestamp except weekends and holidays)
- outliers methods: https://papers.ssrn.com/sol3/papers.cfm?abstract_id=886204
- compare multiple sources (differnet brokers)
- outside of trading hours removed
- excess return in a specific time len (example daily, 5 min, ...)
- still more like: https://s3-us-west-2.amazonaws.com/tick-data-s3/pdf/Tick_Data_Filtering_White_Paper.pdf

-----
more for feature selection:

* normalizing features (not just data) without leak!!!
* dealing with small movement of price, like pips!!!
* noise removing with fourier!!
"""


def find_missing_timestamp(data: pd.DataFrame, freq="T", exclude_weekends=True):
    """
    return missing timestamp, example if minutes and there are some missing minutes, it returns those
    :param data: with datetime index
    :param freq: frequency that search for missing timestamp will be applied
    :return:
    """
    assert data.index.name == "Timestamp", "data needs to have timestamp index"
    data["__temp"] = 1

    missing_timestamps = data[["__temp"]].resample(freq).mean().isna()
    missing_timestamps = missing_timestamps[missing_timestamps["__temp"]]

    if exclude_weekends:
        missing_timestamps = missing_timestamps[missing_timestamps.index.dayofweek != 6]
        missing_timestamps = missing_timestamps[missing_timestamps.index.dayofweek != 5]

    _ = data.pop("__temp")

    return missing_timestamps


def check_close_match_open_next_bar(ohlc: pd.DataFrame):
    assert "Close" in ohlc.columns, "Close missing in colomns"
    assert "Open" in ohlc.columns, "Open missing in colomns"

    temp_col = "Next_Open"
    ohlc[temp_col] = ohlc["Open"].shift(-1)

    none_match = ohlc[ohlc[temp_col] != ohlc["Close"]]

    _ = ohlc.pop(temp_col)
    return none_match.index


def check_high_low_values(ohlc: pd.DataFrame):
    """ high > open, close > low"""
    assert "Close" in ohlc.columns, "Close missing in colomns"
    assert "Open" in ohlc.columns, "Open missing in colomns"
    assert "High" in ohlc.columns, "High missing in colomns"
    assert "Low" in ohlc.columns, "Low missing in colomns"

    close_problems = ohlc[(ohlc["Close"] > ohlc["High"]) | (ohlc["Close"] < ohlc["Low"])]
    open_problems = ohlc[(ohlc["Open"] > ohlc["High"]) | (ohlc["Open"] < ohlc["Low"])]

    return close_problems, open_problems


def remove_violating_threshold(data: pd.DataFrame, low=0.5, high=1.3):
    """
    removing data lower than low or higher than high
    :return:
    """
    lower_low = (data < low).sum(axis=1) > 0
    higher_high = (data > high).sum(axis=1) > 0

    violated = data[lower_low | higher_high]
    in_bound = data[~lower_low & ~higher_high]

    return in_bound, violated


def remove_weekends_bars(data: pd.DataFrame):
    assert data.index.name == "Timestamp", "data needs to have timestamp index"

    saturday_filter = data.index.dayofweek == 5
    sunday_filter = data.index.dayofweek == 6

    weekends = data[saturday_filter | sunday_filter]
    no_weekends = data[~saturday_filter & ~sunday_filter]

    return no_weekends, weekends


